/**
 *
 * @file starpu/codelet_ztstrf.c
 *
 * @copyright 2009-2014 The University of Tennessee and The University of
 *                      Tennessee Research Foundation. All rights reserved.
 * @copyright 2012-2019 Bordeaux INP, CNRS (LaBRI UMR 5800), Inria,
 *                      Univ. Bordeaux. All rights reserved.
 *
 ***
 *
 * @brief Chameleon ztstrf StarPU codelet
 *
 * @version 0.9.2
 * @comment This file has been automatically generated
 *          from Plasma 2.5.0 for CHAMELEON 0.9.2
 * @author Hatem Ltaief
 * @author Jakub Kurzak
 * @author Mathieu Faverge
 * @author Emmanuel Agullo
 * @author Cedric Castagnede
 * @author Lucas Barros de Assis
 * @date 2014-11-16
 * @precisions normal z -> c d s
 *
 */
#include "chameleon_starpu.h"
#include "runtime_codelet_z.h"

#if !defined(CHAMELEON_SIMULATION)
static void cl_ztstrf_cpu_func(void *descr[], void *cl_arg)
{
    CHAMELEON_starpu_ws_t *d_work;
    int m;
    int n;
    int ib;
    int nb;
    CHAMELEON_Complex64_t *U;
    int ldU;
    CHAMELEON_Complex64_t *A;
    int ldA;
    CHAMELEON_Complex64_t *L;
    int ldL;
    int *IPIV;
    CHAMELEON_Complex64_t *WORK;
    int ldWORK;
    cham_bool_t check_info;
    int iinfo;
    RUNTIME_sequence_t *sequence;
    RUNTIME_request_t *request;
    int info = 0;

    U = (CHAMELEON_Complex64_t *)STARPU_MATRIX_GET_PTR(descr[0]);
    A = (CHAMELEON_Complex64_t *)STARPU_MATRIX_GET_PTR(descr[1]);
    L = (CHAMELEON_Complex64_t *)STARPU_MATRIX_GET_PTR(descr[2]);
    WORK = (CHAMELEON_Complex64_t *)STARPU_MATRIX_GET_PTR(descr[3]);
    ldU = STARPU_MATRIX_GET_LD( descr[0] );
    ldA = STARPU_MATRIX_GET_LD( descr[1] );
    ldL = STARPU_MATRIX_GET_LD( descr[2] );
    starpu_codelet_unpack_args(cl_arg, &m, &n, &ib, &nb,
                               &IPIV, &d_work, &ldWORK, &check_info, &iinfo,
                               &sequence, &request);

    CORE_ztstrf(m, n, ib, nb, U, ldU, A, ldA, L, ldL, IPIV, WORK, ldWORK, &info);

    if ( (sequence->status == CHAMELEON_SUCCESS) && (info != 0) ) {
        RUNTIME_sequence_flush( NULL, sequence, request, iinfo+info );
    }
}
#endif /* !defined(CHAMELEON_SIMULATION) */

/*
 * Codelet definition
 */
CODELETS_CPU(ztstrf, 4, cl_ztstrf_cpu_func)

/**
 *
 * @ingroup INSERT_TASK_Complex64_t
 *
 *  CORE_ztstrf computes an LU factorization of a complex matrix formed
 *  by an upper triangular NB-by-N tile U on top of a M-by-N tile A
 *  using partial pivoting with row interchanges.
 *
 *  This is the right-looking Level 2.5 BLAS version of the algorithm.
 *
 *******************************************************************************
 *
 * @param[in] M
 *         The number of rows of the tile A.  M >= 0.
 *
 * @param[in] N
 *         The number of columns of the tile A.  N >= 0.
 *
 * @param[in] IB
 *         The inner-blocking size.  IB >= 0.
 *
 * @param[in] NB
 *
 * @param[in,out] U
 *         On entry, the NB-by-N upper triangular tile.
 *         On exit, the new factor U from the factorization
 *
 * @param[in] ldU
 *         The leading dimension of the array U.  ldU >= max(1,NB).
 *
 * @param[in,out] A
 *         On entry, the M-by-N tile to be factored.
 *         On exit, the factor L from the factorization
 *
 * @param[in] ldA
 *         The leading dimension of the array A.  ldA >= max(1,M).
 *
 * @param[in,out] L
 *         On entry, the IB-by-N lower triangular tile.
 *         On exit, the interchanged rows form the tile A in case of pivoting.
 *
 * @param[in] ldL
 *         The leading dimension of the array L.  ldL >= max(1,IB).
 *
 * @param[out] IPIV
 *         The pivot indices; for 1 <= i <= min(M,N), row i of the
 *         tile U was interchanged with row IPIV(i) of the tile A.
 *
 * @param[in,out] WORK
 *
 * @param[in] ldWORK
 *         The dimension of the array WORK.
 *
 * @param[out] INFO
 *
 *******************************************************************************
 *
 * @retval CHAMELEON_SUCCESS successful exit
 * @retval <0 if INFO = -k, the k-th argument had an illegal value
 * @retval >0 if INFO = k, U(k,k) is exactly zero. The factorization
 *              has been completed, but the factor U is exactly
 *              singular, and division by zero will occur if it is used
 *              to solve a system of equations.
 *
 */
void INSERT_TASK_ztstrf( const RUNTIME_option_t *options,
                         int m, int n, int ib, int nb,
                         const CHAM_desc_t *U, int Um, int Un, int ldU,
                         const CHAM_desc_t *A, int Am, int An, int ldA,
                         const CHAM_desc_t *L, int Lm, int Ln, int ldL,
                         int *IPIV,
                         cham_bool_t check_info, int iinfo )
{
    (void)nb;
    struct starpu_codelet *codelet = &cl_ztstrf;
    void (*callback)(void*) = options->profiling ? cl_ztstrf_callback : NULL;
    CHAMELEON_starpu_ws_t *d_work = (CHAMELEON_starpu_ws_t*)(options->ws_host);

    CHAMELEON_BEGIN_ACCESS_DECLARATION;
    CHAMELEON_ACCESS_RW(U, Um, Un);
    CHAMELEON_ACCESS_RW(A, Am, An);
    CHAMELEON_ACCESS_W(L, Lm, Ln);
    CHAMELEON_END_ACCESS_DECLARATION;

    starpu_insert_task(
        starpu_mpi_codelet(codelet),
        STARPU_VALUE,    &m,                         sizeof(int),
        STARPU_VALUE,    &n,                         sizeof(int),
        STARPU_VALUE,    &ib,                        sizeof(int),
        STARPU_VALUE,    &nb,                        sizeof(int),
        STARPU_RW,        RTBLKADDR(U, CHAMELEON_Complex64_t, Um, Un),
        STARPU_RW,        RTBLKADDR(A, CHAMELEON_Complex64_t, Am, An),
        STARPU_W,         RTBLKADDR(L, CHAMELEON_Complex64_t, Lm, Ln),
        STARPU_VALUE,    &IPIV,                      sizeof(int*),
        STARPU_SCRATCH,   options->ws_worker,
        STARPU_VALUE,    &d_work,                    sizeof(CHAMELEON_starpu_ws_t *),
        STARPU_VALUE,    &nb,                        sizeof(int),
        STARPU_VALUE,    &check_info,                sizeof(cham_bool_t),
        STARPU_VALUE,    &iinfo,                     sizeof(int),
        STARPU_VALUE,    &(options->sequence),       sizeof(RUNTIME_sequence_t*),
        STARPU_VALUE,    &(options->request),        sizeof(RUNTIME_request_t*),
        STARPU_PRIORITY,  options->priority,
        STARPU_CALLBACK,  callback,
#if defined(CHAMELEON_CODELETS_HAVE_NAME)
        STARPU_NAME, "ztstrf",
#endif
        0);
    (void)ldL;
    (void)ldA;
    (void)ldU;
}
